package jTrolog.engine;

import jTrolog.terms.Term;
import jTrolog.terms.Var;

import java.util.*;
import java.lang.reflect.Array;

/**
 * @author ivar.orstavik@hist.no
 */
class LinkTable {

    public static int maxVarCount = 100;
    public static int mapSize = 200;

    private ExpandableTermMap linkedTerms = new ExpandableTermMap(64);
    private ExpandableIntMap linkedCtx = new ExpandableIntMap(64);
    //todo fix TrashCan to handle variable rows and columns
    private TrashCanIntToIntInt trashCanB = new TrashCanIntToIntInt(mapSize);

    public Term getTerm(Var v, int vCtx) {
        return linkedTerms.get(vCtx, v.getNrInStruct());
    }

    public int getCtx(Var v, int vCtx) {
        return linkedCtx.get(vCtx, v.getNrInStruct());
    }

    public void put(Var v, int vCtx, Term link, int linkCtx, int execCtx) {
        linkedTerms.put(vCtx, v.getNrInStruct(), link);
        linkedCtx.put(vCtx, v.getNrInStruct(), linkCtx);
        trashCanB.putLast(execCtx, v.getNrInStruct(), vCtx);
    }

    void gcOldLinks(int owner) {
        for (int i = 0; trashCanB.contains(owner, i); i++) {
            int trashThisLink = trashCanB.get1(owner, i);
            int trashThisCtx = trashCanB.get2(owner, i);
            linkedTerms.put(trashThisCtx, trashThisLink, null);
            linkedCtx.put(trashThisCtx, trashThisLink, 0);
        }
    }

    void resetGarbage(int owner) {
        trashCanB.reset(owner);
    }

    Iterator getKeysForCp(final int owner) {
        return new Iterator(){

            int ctx = owner;
            int i = 0;

            public boolean hasNext() {
                return trashCanB.contains(ctx, i);
            }

            public Object next() {
                if (!hasNext())
                    throw new IndexOutOfBoundsException();
                int link = trashCanB.get1(ctx, i++);
                int lCtx = trashCanB.get2(ctx, i++);
                return lCtx * maxVarCount + link;
            }

            public void remove() {
                throw new UnsupportedOperationException("dont delete on me");
            }
        };
    }

    private static class ExpandableTermMap {

        private Object[] positiveRows;
        private Object[] negativeRows;

        public ExpandableTermMap(int rowSize) {
            positiveRows = new Object[rowSize];
            negativeRows = new Object[rowSize];
        }

        public void put(int rowNr, int columnNr, Term value) {
            if (rowNr >= positiveRows.length){
                positiveRows = (Object[]) doubleRowSize(positiveRows, positiveRows.length*2);
                negativeRows = (Object[]) doubleRowSize(negativeRows, negativeRows.length*2);
            }
            Object[] rows = positiveRows;
            if (rowNr < 0) {
                rowNr *= -1;
                rows = negativeRows;
            }
            Term[] row = (Term[]) rows[rowNr];
            if (rows[rowNr] == null)
                row = new Term[columnNr+2];                                //just two variables is lean and mean
            else if (columnNr >= row.length)
                row = (Term[]) doubleRowSize(rows[rowNr], columnNr*4);     //if its worth doubling, its worth doubling
            row[columnNr] = value;
            rows[rowNr] = row;
        }

        public Term get(int rowNr, int columnNr){
            if (rowNr >= positiveRows.length)
                return null;
            Object[] rows = this.positiveRows;
            if (rowNr < 0) {
                rowNr *= -1;
                rows = negativeRows;
            }
            return rows[rowNr] == null || columnNr >= ((Term[]) rows[rowNr]).length ?
                    null :
                    ((Term[]) rows[rowNr])[columnNr];
        }

        static Object doubleRowSize(Object array, int newSize) {
            Object newArray = Array.newInstance(array.getClass().getComponentType(), newSize);
            System.arraycopy(array, 0, newArray, 0, Array.getLength(array));
            return newArray;
        }
    }

    private static class ExpandableIntMap {

        private Object[] positiveRows;
        private Object[] negativeRows;

        public ExpandableIntMap(int rowSize) {
            positiveRows = new Object[rowSize];
            negativeRows = new Object[rowSize];
        }

        public void put(int rowNr, int columnNr, int value) {
            if (rowNr >= positiveRows.length){
                positiveRows = (Object[]) ExpandableTermMap.doubleRowSize(positiveRows, positiveRows.length*2);
                negativeRows = (Object[]) ExpandableTermMap.doubleRowSize(negativeRows, negativeRows.length*2);
            }
            Object[] rows = this.positiveRows;
            if (rowNr < 0) {
                rowNr *= -1;
                rows = negativeRows;
            }
            int[] row = (int[]) rows[rowNr];
            if (rows[rowNr] == null)
                row = new int[columnNr+2];                                //just two variables is lean and mean
            else if (columnNr >= row.length)
                row = (int[]) ExpandableTermMap.doubleRowSize(rows[rowNr], columnNr*4);     //if its worth doubling, its worth doubling
            row[columnNr] = value;
            rows[rowNr] = row;
        }

        public int get(int rowNr, int columnNr){
            Object[] rows = this.positiveRows;
            if (rowNr < 0) {
                rowNr *= -1;
                rows = negativeRows;
            }
            return rows[rowNr] == null ?
                    0 :
                    ((int[]) rows[rowNr])[columnNr];
        }
    }

    private static class TrashCanIntToIntInt {
        static int columnSize = 8;
        Object[] rows1;
        Object[] rows2;

        public TrashCanIntToIntInt(int rowSize) {
            rows1 = new Object[rowSize];
            rows2 = new Object[rowSize];
        }

        public void putLast(int rowNr, int value1, int value2) {
            Object[] rows1 = this.rows1;
            Object[] rows2 = this.rows2;
            int[] row1 = (int[]) rows1[rowNr];
            int[] row2 = (int[]) rows2[rowNr];

            if (row1 == null) {
                rows1[rowNr] = row1 = new int[columnSize];
                rows2[rowNr] = row2 = new int[columnSize];
                row1[0] = value1;
                row2[0] = value2;
                return;
            }

            //find first vacant slot (containing value 0)
            for (int i = 0; i < row1.length; i++) {
                if (row1[i] == 0){
                    row1[i] = value1;
                    row2[i] = value2;
                    return;
                }
            }
            //no vacant slots, double and add
            row1 = new int[row1.length * 2]; //likely to be 8 * 2
            row2 = new int[row2.length * 2]; //likely to be 8 * 2
            System.arraycopy(rows1[rowNr], 0, row1, 0, ((int[]) rows1[rowNr]).length);
            System.arraycopy(rows2[rowNr], 0, row2, 0, ((int[]) rows2[rowNr]).length);
            row1[((int[]) rows1[rowNr]).length] = value1;
            row2[((int[]) rows2[rowNr]).length] = value2;
            rows1[rowNr] = row1;
            rows2[rowNr] = row2;
        }

        public boolean contains(int ctx, int i) {
            int[] row = (int[]) rows1[ctx];
            if (row == null)
                return false;
            if (row.length <= i)
                return false;
            return row[i] != 0;
        }

        public void reset(int ctx) {
            if (rows1[ctx] == null)
                return;
            Arrays.fill((int[]) rows1[ctx], 0);
            Arrays.fill((int[]) rows2[ctx], 0);
        }

        public int get1(int ctx, int i) {
            return ((int[]) rows1[ctx])[i];
        }
        public int get2(int ctx, int i) {
            return ((int[]) rows2[ctx])[i];
        }
    }
}
