package jTrolog.engine;

import jTrolog.terms.Int;
import jTrolog.terms.Struct;
import jTrolog.terms.StructAtom;

import java.util.*;

/**
 * Map of Prolog operators.
 * A LinkedHashMap is set up to store each registered operator as:
 *   'name type' -> priority
 *
 * @author ivar.orstavik@hist.no
 */
class OperatorTable implements java.io.Serializable {

    /** current known operators */
    private HashMap operatorMap = new LinkedHashMap();

    /**
     * Creates a new operator. If the operator is already provided,
     * it replaces it with the new one
     */
    protected void addOperator(String name,String type,int prio) {
        if (prio >= Prolog.OP_LOW && prio <= Prolog.OP_HIGH)
            operatorMap.put(name + " " + type, prio);
    }

    /**
     * @return the priority of an operator (0 if the operator is not defined).
     */
    int getOperatorPriority(String name,String type) {
        Integer prio = (Integer) operatorMap.get(name + " " + type);
        return (prio == null) ? 0 : prio.intValue();
    }

    /**
     *  @return a list of Struct Objects representing the operators currently defined,
     *          ordered by insertionOrder
     */
    Iterator getAllOperators() {
        return new OperatorIterator(operatorMap);
    }

    private static class OperatorIterator implements Iterator {
        Iterator underlyingIT;
        Map underlyingMap;

        public OperatorIterator(Map map) {
            this.underlyingIT = map.keySet().iterator();
            this.underlyingMap = map;
        }

        public boolean hasNext() {
            return underlyingIT.hasNext();
        }

        public Object next() {
            String nameType = (String) underlyingIT.next();
            String name = nameType.substring(0, nameType.lastIndexOf(' '));
            String type = nameType.substring(nameType.lastIndexOf(' ')+1);
            int prio = ((Integer) underlyingMap.get(nameType)).intValue();
            return new Struct("op", new Int(prio),new StructAtom(type), new StructAtom(name));
        }

        public void remove() {
            throw new UnsupportedOperationException("can't delete on the operator iterator");
        }
    }

}