package jTrolog.terms;

import jTrolog.errors.InvalidTermException;
import jTrolog.terms.Term;

/**
 * This class represents a variable term. Variables are identified by a name
 * (which must starts with an upper case letter) or the anonymous ('_') name.
 *
 * @see Term
 */
public class Var implements Term {

    public final static String ANY = "_".intern();

    private String name;
    private int nrInStruct = 0;

    /**
     * needed to implement Wrapper Objects. Do not use to create ANY Vars
     */
    Var() {
    }

    /**
     * Creates a variable identified by a name.
     *
     * @param n varaible name
     *          if n is "_" the variable is anonymous.
     * @param nr its variable number of its root parent Struct.
     */
    public Var(String n, int nr) throws InvalidTermException {
        name = n.intern();
        nrInStruct = nr;
    }

    public boolean equals(Object t) {
        return t instanceof Var && getNrInStruct() == ((Var) t).getNrInStruct();
    }

    public int hashCode() {
        return getNrInStruct();
    }

    /**
     * Tests if this variable is ANY
     */
    public boolean isAnonymous() {
        return name == ANY;
    }

    /**
     * Gets the string representation of this variable.
     */
    public String toString() {
        return name;
    }

    public String toStringSmall() {
        return name;
    }

    public int getNrInStruct() {
        return nrInStruct;
    }
}