package jTrolog.engine;

import fit.Fixture;
import jTrolog.errors.PrologException;
import jTrolog.parser.Parser;
import jTrolog.terms.Term;

import java.io.PrintStream;
import java.io.ByteArrayOutputStream;

public class EngineFixture extends Fixture {
    private Prolog engine = new Prolog();
    private String query;
    private Solution result;
    private String variable;
    private String output;
    private String exception;

    /* Registers */

    public void theory(String program) throws PrologException {
        engine.setTheory(program);
    }

    public void query(String problem) {
        query = problem;
    }

    public void variable(String name) {
        variable = name;
    }

    /* Meters */

    public boolean hasSolution(){
        try {
            result = engine.solve(query);
            return result.success();
        } catch (Throwable e) {
            exception = e.getMessage();
            return false;
        }
    }

    public boolean hasSolutionWithOutput() {
        PrintStream orig = System.out;
        ByteArrayOutputStream listener = new ByteArrayOutputStream();
        System.setOut(new PrintStream(listener));
        output = "";
        try {
            result = engine.solve(query);
            return result.success();
        } catch (Throwable e) {
            exception = e.getMessage();
            return false;
        } finally{
            output = listener.toString();
            System.setOut(orig);
        }
    }

    public boolean hasAnotherSolution() {
        try {
            result = engine.solveNext();
            return result.success();
        } catch (Throwable throwable) {
            exception = throwable.getMessage();
            return false;
        }
    }

    public boolean hasAnotherSolutionWithOutput() {
        PrintStream orig = System.out;
        ByteArrayOutputStream listener = new ByteArrayOutputStream();
        System.setOut(new PrintStream(listener));
        output = "";
        try {
            result = engine.solveNext();
            return result.success();
        } catch (Throwable e) { // NoSolutionException, NoMorePrologSolutions
            exception = e.getMessage();
            return false;
        } finally{
            output = listener.toString();
            System.setOut(orig);
        }
    }

    public Term binding() throws PrologException {
        return result.getBinding(variable);
    }

    public String output() {
        return output;
    }

    public String exception() {
        if (exception == null)
            return "no exception";
        return exception;
    }

    public String getTheory() {
        return engine.getTheory();
    }

    /* Utilities */

    public Object parse(String s, Class type) throws Exception {
        if (type.equals(jTrolog.terms.Term.class))
            return BindingsTable.wrapWithID(new Parser(s).nextTerm(false),0);
        return super.parse(s, type);
    }
}