package jTrolog.engine;

import junit.framework.TestCase;
import jTrolog.errors.NoMorePrologSolutions;
import jTrolog.terms.Term;
import jTrolog.parser.Parser;

public class DCGTestCaseIVar extends TestCase {

    private static String comment = "/* hello */";

    public void testDCG4() throws Throwable, NoMorePrologSolutions {
        String th =
                " star_comment(CommentText) -->  ['/', '*'], star_comment_body(CommentText). " +
                " star_comment_body([]) -->  ['*', '/']. " +
                " star_comment_body([A|Out]) -->  [A], star_comment_body(Out).";

        Prolog eng = new Prolog("jTrolog.lib.DCGLibrary");
        eng.solve("set_prolog_flag(double_quotes,chars).");
        Solution res2 = eng.solve("current_prolog_flag(double_quotes,X).");
        eng.addTheory(th);
        eng.solve("dcg_preprocess.");
        Solution res = eng.solve("star_comment(X, \""+ DCGTestCaseIVar.comment + "\", Y).");
        Term varValue = res.getBinding("X");
        assertEquals(varValue, new Parser("[' ', h ,e, l, l, o, ' ']").nextTerm(false));
    }

    public void testDCG5() throws Throwable,  NoMorePrologSolutions {
        String th =
                "star_comment(CommentText) --> " +
                        "['/', '*'], star_comment_body(CommentText). " +
                "star_comment_body(CommentText) -*-> " +
                        "[CommentText, '\\*/'].";

        Prolog eng = new Prolog("jTrolog.lib.DCGLibrary");
        eng.addTheory(th);
        eng.solve("dcg_preprocess.");

        eng.solve("set_prolog_flag(double_quotes,chars).");
        Solution res2 = eng.solve("current_prolog_flag(double_quotes,X).");

        Solution res = eng.solve("star_comment(X, \""+ DCGTestCaseIVar.comment + "\", Y).");
        Term varValue = res.getBinding("X");
        assertEquals(varValue, new Parser("[' ', h ,e, l, l, o, ' ']").nextTerm(false));
    }

    public void testDCG6() throws Throwable, NoMorePrologSolutions {
        String th = "star_comment(CommentText) -*-> ['/\\*', CommentText, '\\*/'].";

        Prolog eng = new Prolog("jTrolog.lib.DCGLibrary");
        eng.solve("set_prolog_flag(double_quotes,chars).");
        Solution res2 = eng.solve("current_prolog_flag(double_quotes,X).");
        eng.addTheory(th);
        eng.solve("dcg_preprocess.");
        Solution res = eng.solve("star_comment(X, \""+ DCGTestCaseIVar.comment + "\", Y).");
        Term varValue = res.getBinding("X");
        assertEquals(varValue, new Parser("[' ', h ,e, l, l, o, ' ']").nextTerm(false));
    }

    public void testDCG7() throws Throwable, NoMorePrologSolutions {
        String th = "star_comment_start(CommentSigns) -&-> [CommentSigns, '/\\*'].";

        Prolog eng = new Prolog("jTrolog.lib.DCGLibrary");
        eng.solve("set_prolog_flag(double_quotes,chars).");
        Solution res2 = eng.solve("current_prolog_flag(double_quotes,X).");
        eng.addTheory(th);
        eng.solve("dcg_preprocess.");
        Solution res = eng.solve("star_comment_start(X, \""+ DCGTestCaseIVar.comment + "\", Y).");
        Term varValue = res.getBinding("X");
        assertEquals(varValue, new Parser("['/', '*']").nextTerm(false));
    }

    public void testDCG1() throws Throwable, NoMorePrologSolutions {
        String theory =
            "star_comment([C1,C2|C1s])  --> " +
                "[C1,C2], " +
                "{is_slash(C1), is_star(C2), !}, star_comment_body(C1s). ";

        Prolog engine = new Prolog("jTrolog.lib.DCGLibrary");
        engine.addTheory(theory);
        Solution result = engine.solve("dcg_preprocess.");

        String shouldBe =
        "star_comment([C1|[C2|C1s]],DCG_P_0,DCG_P_2):-\n" +
        "\t','(dcg_accummulator_with_stop([C1, C2],DCG_P_0,DCG_P_1),','(','(is_slash(C1),','(is_star(C2),!)),star_comment_body(C1s,DCG_P_1,DCG_P_2))).\n\n";
        assertEquals(engine.getTheory(), shouldBe);
    }

    public void testDCG2() throws Throwable, NoMorePrologSolutions {
        String theory =
            "b_list([H|T]) --> " +
                "b(H), !, " +
                "b_list(T). " +
            "b_list([], Stop, Stop). ";

        Prolog engine = new Prolog("jTrolog.lib.DCGLibrary");
        engine.addTheory(theory);
        Solution result = engine.solve("dcg_preprocess.");

        String shouldBe =
        "b_list([H|T],DCG_P_0,DCG_P_2):-\n" +
        "\t','(b(H,DCG_P_0,DCG_P_1),','(!,b_list(T,DCG_P_1,DCG_P_2))).\n\n" +
        "b_list([],Stop,Stop).\n\n";
        assertEquals(engine.getTheory(), shouldBe);
    }

    public void testDCG3() throws Throwable, NoMorePrologSolutions {
        String theory =
            "star_comment(['/','*' | Body])  --> " +
                "['/','*'],!, " +
                "[Body, '*', '/'].";

        Prolog engine = new Prolog("jTrolog.lib.DCGLibrary");
        engine.addTheory(theory);
        Solution result = engine.solve("dcg_preprocess.");

        String shouldBe =
        "star_comment(['/'|['*'|Body]],DCG_P_0,DCG_P_2):-\n" +
        "\t','(dcg_accummulator_with_stop(['/', '*'],DCG_P_0,DCG_P_1),','(!,dcg_accummulator_with_skip_and_stop(Body,['*', '/'],DCG_P_1,DCG_P_2))).\n\n".intern();
        assertEquals(engine.getTheory(), shouldBe);
    }

}
