package jTrolog.engine;

import junit.framework.TestCase;
import jTrolog.terms.Int;
import jTrolog.terms.StructList;
import jTrolog.terms.Term;

public class EngineTestCase extends TestCase {

    public void testCut() throws Throwable {
        Prolog p = new Prolog();
        p.setTheory("a(X) :- X is 1. "
                + "a(X) :- X is 2, !, Y is 3. " + "a(X) :- X is 3. ");

        Solution res = p.solve("a(X).");
        assertTrue(res.success());
        assertEquals(new Int(1), res.getBinding("X"));

        assertTrue(p.hasOpenAlternatives());

        res = p.solveNext();
        assertTrue(res.success());
        assertEquals(new Int(2), res.getBinding("X"));

        assertFalse(p.hasOpenAlternatives());
    }

    public void testVariableLessOrEqualThan() throws Throwable {
        Prolog eng = new Prolog();
        Solution res = eng.solve("'@=<'(X, X). ");
        assertTrue(res.success());
    }

    public void testNot() throws Throwable {
        Prolog p = new Prolog();
        p.setTheory("a :- true. ");
        Solution res = p.solve("not(a). ");

        assertFalse(res.success());
    }

    public void testListOperations() throws Throwable {
        // // Solution res = new Prolog().solve("=..(foo(a,b,c), [foo, a|X]).
        // ");
        // // Solution res = new Prolog().solve("=..(foo(a,b), [Name|Args]),
        // compound(foo(a,b)), length(Args, 2). ");
        Prolog eng = new Prolog();
        Solution res = eng.solve("'=..'(foo(X, b), [foo, a, Y]). ");

        assertTrue(res.success());
        assertEquals("a", res.getBinding("X").toString());
        assertEquals("b", res.getBinding("Y").toString());
    }

    public void testElement() throws Throwable {
        Solution res = new Prolog().solve("element(1, [X,b,c], Y).");

        assertTrue(res.success());
        assertEquals("X", res.getBinding("Y").toString());
    }

    public void testClauseRetrieving() throws Throwable {
        String theory = ":- dynamic(cat/0). " + " cat. " +

        ":- dynamic(dog/0). " + "dog :- true. " +

        "elk(X) :- moose(X). " +

        ":- dynamic(legs/2). " + "legs(A, 6) :- insect(A). "
                + "legs(A, 7) :- A, call(A). " +

                ":- dynamic(insect/1). " + "insect(ant). " + "insect(bee). ";

        Prolog p = new Prolog();
        p.addTheory(theory);

        // Solution res = p.solve("member(X, [a,b,c]).");
//		Solution res = p.solve("clause(_, B).");
        // Solution res = p.solve("clause(legs(A, 6), insect(f(A))).");
         Solution res = p.solve("clause(legs(I, 6), Body).");
        // Solution res = p.solve("L=[], '$find'(cat, L). ");
        // clause(H,B) :- L=[],'$find'(H,L),member((':-'(H,B)),L). \n
        assertTrue(res.success());
        assertTrue(p.hasOpenAlternatives());

    }

    public void testCreationOfBindingsTables() throws Throwable {
        Prolog eng = new Prolog();
        Solution res = eng.solve("'=..'(foo(a,b,c),[foo|Args]).");

        assertTrue(res.success());
        Term binding = res.getBinding("Args");
        assertTrue(binding instanceof StructList);
        assertEquals("[a, b, c]", binding.toString());
    }

    public void testFunctorFoo() throws Throwable {
        Prolog eng = new Prolog();
        Solution res = eng.solve("functor(foo(a,b,c), foo, 3).");
        assertTrue(res.success());
    }

    public void testNonVar() throws Throwable {
        Solution res = new Prolog().solve("foo = Foo, nonvar(Foo).");

        assertTrue(res.success());
        assertEquals("foo", res.getBinding("Foo").toString());
    }

    public void testLength() throws Throwable {
        Prolog eng = new Prolog();
        Solution res = eng.solve("lengthN([a,b,c], 3).");

        assertTrue(res.success());
    }

    public void testRetract() throws Throwable {
        String theory = " :- dynamic(legs/2). " +
                "legs(A, 4) :- animal(A). "
                + "legs(octopus, 8). " +
                "legs(A, 6) :- insect(A). "
                + "legs(spider, 8). " +
                "legs(B, 2) :- bird(B). " +

                ":- dynamic(insect/1). " + "insect(ant). " + "insect(bee). " +

                " :- dynamic(foo/1). " + "foo(X) :- call(X), call(X). "
                + " foo(X) :- call(X) -> call(X). ";

        Prolog p = new Prolog();
        p.setTheory(theory);

        Solution res = p.solve("retract(insect(X)).");

        assertTrue(res.success());
        res = p.solveNext();
        assertTrue(res.success());
    }

    public void testCurrentPredicate() throws Throwable {
        String theory = ":- dynamic(cat/0). " + " cat. " +
                        ":- dynamic(dog/0). " + "dog :- true. " +

                        "elk(X) :- moose(X). " +

                        ":- dynamic(legs/2). " + "legs(A, 6) :- insect(A). "
                        + "legs(A, 7) :- A, call(A). " +

                        ":- dynamic(insect/1). " + "insect(ant). " + "insect(bee). ";

        Prolog p = new Prolog();
        p.setTheory(theory);

        Solution res = p.solve("current_predicate(current_predicate/1). ");

        System.out.println(res);
    }

    public void testFindAll() throws Throwable {
        Solution res = new Prolog().solve("findall(X, (X=1; X=2), S). ");
        // Solution res = new Prolog().solve("L = [], '$append'(1, L). ");
    }
}
