package jTrolog.engine;

import junit.framework.TestCase;
import jTrolog.errors.InvalidLibraryException;
import jTrolog.terms.Int;
import jTrolog.terms.StructAtom;

/**
 *
 * @author janerist@stud.ntnu.no
 */
public class PrologProgramsTestCase extends TestCase {

    private static String riddle =
            "next_to(X,Y,List) :-\n" +
            "    iright(X,Y,List).\n" +
            "next_to(X,Y,List) :-\n" +
            "    iright(Y,X,List).\n" +
            "einstein(Houses,Fish_Owner) :-\n" +
            "    '='(Houses, [[house,norwegian,_,_,_,_],_,[house,_,_,_,milk,_],_,_]),\n" +
            "    member([house,brit,_,_,_,red],Houses),\n" +
            "    member([house,swede,dog,_,_,_],Houses),\n" +
            "    member([house,dane,_,_,tea,_],Houses),\n" +
            "    iright([house,_,_,_,_,green],[house,_,_,_,_,white],Houses),\n" +
            "    member([house,_,_,_,coffee,green],Houses),\n" +
            "    member([house,_,bird,pallmall,_,_],Houses),\n" +
            "    member([house,_,_,dunhill,_,yellow],Houses),\n" +
            "    next_to([house,_,_,dunhill,_,_],[house,_,horse,_,_,_],Houses),\n" +
            "    member([house,_,_,_,milk,_],Houses),\n" +
            "    next_to([house,_,_,marlboro,_,_],[house,_,cat,_,_,_],Houses),\n" +
            "    next_to([house,_,_,marlboro,_,_],[house,_,_,_,water,_],Houses),\n" +
            "    member([house,_,_,winfield,beer,_],Houses),\n" +
            "    member([house,german,_,rothmans,_,_],Houses),\n" +
            "    next_to([house,norwegian,_,_,_,_],[house,_,_,_,_,blue],Houses),\n" +
            "    member([house,Fish_Owner,fish,_,_,_],Houses).\n" +
            "iright(L,R,[L,R|_]).\n" +
            "iright(L,R,[_|Rest]) :-\n" +
            "    iright(L,R,Rest). ";

    public void testEinsteinsRiddle() throws Throwable, InvalidLibraryException {
        Prolog p = new Prolog();
        p.addTheory(riddle);
//        Solution res = p.solve("spy.");
        long start = System.currentTimeMillis();
        Solution res = p.solve("einstein(Y,X).");
        System.out.println("einstein time= " + (System.currentTimeMillis()-start));
        assertTrue(res.success());
        assertEquals(
                "einstein([[house, norwegian, cat, dunhill, water, yellow], " +
                        "[house, dane, horse, marlboro, tea, blue], " +
                        "[house, brit, bird, pallmall, milk, red], " +
                        "[house, german, fish, rothmans, coffee, green], " +
                        "[house, swede, dog, winfield, beer, white]], " +
                        "german)",res.getSolution().toString());
        assertEquals(res.getBinding("X"), new StructAtom("german"));
    }

    public void testTowersOfHanor() throws Throwable {
        String theory =
                "move(1,X,Y,_) :-  "+
                "write('Move top disk from '), "+
                "write(X), "+
                "write(' to '), "+
                "write(Y), "+
                "nl. "+
                "move(N,X,Y,Z) :- "+
                "N>1, "+
                "M is N-1, "+
                "move(M,X,Z,Y), "+
                "move(1,X,Y,_), "+
                "move(M,Z,Y,X).  ";

        Prolog engine = new Prolog();
        engine.setTheory(theory);

        Solution info = engine.solve("move(3, left, right, center).");

        assertTrue(info.success());

    }

    public void testReverseList() throws Throwable {
        Prolog engine = new Prolog();
        Solution info = engine.solve("reverse([a,b,c], [c,b,a]).");
        assertTrue(info.success());
        info = engine.solve("reverse([a,b,c], [b,c,a]).");
        assertFalse(info.success());
    }


    public void testChessQueen() throws Throwable {
        String theory = "solve_a(P) :- "+
         "perm([1,2,3,4],P), "+
         "combine([1,2,3,4],P,S,D),"+
         "all_diff(S),"+
         "all_diff(D). "+

        "combine([X1|X],[Y1|Y],[S1|S],[D1|D]) :-" +
             "S1 is X1 +Y1,"+
             "D1 is X1 - Y1,"+
             "combine(X,Y,S,D). "+
        "combine([],[],[],[]). "+

        "all_diff([X|Y]) :-  \\+member(X,Y), all_diff(Y). "+
        "all_diff([X]). " +

        "perm([X|Y],Z) :- perm(Y,W), takeout(X,Z,W).  " +
        "perm([],[]). " +

        "takeout(X,[X|R],R). " +
        "takeout(X,[F|R],[F|S]) :- takeout(X,R,S). ";

        Prolog engine = new Prolog();
        engine.setTheory(theory);

        long start = System.currentTimeMillis();
        Solution res = engine.solve("solve_a(X). ");
        System.out.println("queens time= " + (System.currentTimeMillis()-start));

        assertTrue(res.success());
        assertTrue(engine.hasOpenAlternatives());
    }

    public void testChangeForADollar() throws Throwable {
        String theory = "change([H,Q,D,N,P]) :- "+
                            "member(H,[0,1,2]),                      /* Half-dollars */ "+
                            "member(Q,[0,1,2,3,4]),                  /* quarters     */ "+
                            "member(D,[0,1,2,3,4,5,6,7,8,9,10]) ,    /* dimes        */ "+
                            "member(N,[0,1,2,3,4,5,6,7,8,9,10,       /* nickels      */ "+
                                           "11,12,13,14,15,16,17,18,19,20]),  "+
                            "S is 50*H + 25*Q +10*D + 5*N, "+
                            "S =< 100, "+
                            "P is 100-S. ";

        Prolog engine = new Prolog();
        engine.setTheory(theory);

        Solution info = engine.solve("change([0,2,3,2,P]).");

        assertTrue(info.success());
        assertEquals(new Int(10), info.getBinding("P"));
    }

    public void testBug() throws Throwable {
        Prolog engine = new Prolog();
        engine.addTheory("go :- fail.\n" +
                "go :- write(1), X = ','(write(cut), !), X, write(2), fail.\n" +
                "go :- write(3).");
        Solution info = null;
        info = engine.solve("go.");
        info = engine.solve("';'('->'((!, fail), true), fail).");
//        Term res = info.getBinding("Y");
        System.out.println("info " + info);
    }
}
