package jTrolog.engine;

import junit.framework.TestCase;
import jTrolog.errors.InvalidLibraryException;
import jTrolog.lib.Library;
import jTrolog.lib.StringLibrary;

import java.util.Iterator;

public class PrologTestCase extends TestCase {

    public void testEngineInitialization() {
        Prolog engine = new Prolog();
        Iterator libs = engine.getCurrentLibraries();
        assertEquals("jTrolog.lib.BuiltIn", ((Library) libs.next()).getName());
        assertEquals("jTrolog.lib.BasicLibrary", ((Library) libs.next()).getName());
        assertEquals("jTrolog.lib.ISOLibrary", ((Library) libs.next()).getName());
        assertEquals("jTrolog.lib.IOLibrary", ((Library) libs.next()).getName());
        assertFalse(libs.hasNext());
    }

    public void testLoadLibraryAsString() throws InvalidLibraryException {
        Prolog engine = new Prolog();
        engine.loadLibrary("jTrolog.lib.StringLibrary");
        Iterator libs = engine.getCurrentLibraries();
        assertEquals("jTrolog.lib.BuiltIn", ((Library) libs.next()).getName());
        assertEquals("jTrolog.lib.BasicLibrary", ((Library) libs.next()).getName());
        assertEquals("jTrolog.lib.ISOLibrary", ((Library) libs.next()).getName());
        assertEquals("jTrolog.lib.IOLibrary", ((Library) libs.next()).getName());
        assertEquals("jTrolog.lib.StringLibrary", ((Library) libs.next()).getName());
        assertFalse(libs.hasNext());
    }

    public void testLoadLibraryAsObject() throws InvalidLibraryException {
        Prolog engine = new Prolog();

        Library stringLibrary = new StringLibrary();
        engine.loadLibrary(stringLibrary);

        Library ioLib = new jTrolog.lib.IOLibrary();
        engine.loadLibrary(ioLib);

        Iterator libs = engine.getCurrentLibraries();
        assertEquals("jTrolog.lib.BuiltIn", ((Library) libs.next()).getName());
        assertEquals("jTrolog.lib.BasicLibrary", ((Library) libs.next()).getName());
        assertEquals("jTrolog.lib.ISOLibrary", ((Library) libs.next()).getName());
        assertEquals("jTrolog.lib.StringLibrary", ((Library) libs.next()).getName());
        assertEquals(ioLib, libs.next());
        assertFalse(libs.hasNext());
    }

    public void testGetLibraryWithName() throws InvalidLibraryException {
        Prolog engine = new Prolog(new String[] {"jTrolog.lib.TestLibrary"});
        Iterator libs = engine.getCurrentLibraries();
        assertEquals("jTrolog.lib.BuiltIn", ((Library) libs.next()).getName());
        assertEquals("TestLibraryName", ((Library) libs.next()).getName());
        assertFalse(libs.hasNext());
    }
}
