package jTrolog.engine;

import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.LinkedList;
import java.util.Arrays;

import junit.framework.TestCase;
import jTrolog.terms.Int;
import jTrolog.terms.Struct;
import jTrolog.terms.StructAtom;
import jTrolog.terms.Term;

/**
 * 
 * @author <a href="mailto:giulio.piancastelli@unibo.it">Giulio Piancastelli</a>
 */
public class StructIteratorTestCase extends TestCase {

    public void testEmptyIterator() {
        Struct list = Struct.emptyList;
        Iterator i = new BindingsTable().structListIterator(list, true);
        assertFalse(i.hasNext());
        try {
            i.next();
            fail();
        } catch (NoSuchElementException expected) {}
    }

    public void testIteratorCount() {
        Term[] argList = new Term[]{new Int(1), new Int(2), new Int(3), new Int(5), new Int(7), Struct.emptyList};
        Struct list = Struct.createStructList(new LinkedList(Arrays.asList(argList)));
        Iterator i = new BindingsTable().structListIterator(list, true);
        int count = 0;
        for (; i.hasNext(); count++)
            i.next();
        assertEquals(5, count);
        assertFalse(i.hasNext());
    }

    public void testMultipleHasNext() {
        Term[] argList = new Term[]{new StructAtom("p"), new StructAtom("q"), new StructAtom("r"), Struct.emptyList};
        Struct list = Struct.createStructList(new LinkedList(Arrays.asList(argList)));
        Iterator i = new BindingsTable().structListIterator(list, true);
        assertTrue(i.hasNext());
        assertTrue(i.hasNext());
        assertTrue(i.hasNext());
        assertEquals(new StructAtom("p"), i.next());
    }

    public void testMultipleNext() {
        Term[] argList = new Term[]{new Int(0), new Int(1), new Int(2), new Int(3), new Int(5), new Int(7), Struct.emptyList};
        Struct list = Struct.createStructList(new LinkedList(Arrays.asList(argList)));
        Iterator i = new BindingsTable().structListIterator(list, true);
        assertTrue(i.hasNext());
        i.next(); // skip the first term
        assertEquals(new Int(1), i.next());
        assertEquals(new Int(2), i.next());
        assertEquals(new Int(3), i.next());
        assertEquals(new Int(5), i.next());
        assertEquals(new Int(7), i.next());
        // no more terms
        assertFalse(i.hasNext());
        try {
            i.next();
            fail();
        } catch (NoSuchElementException expected) {}
    }

    public void testRemoveOperationNotSupported() {
        Struct list = Struct.createList(".", new Int(1), Struct.emptyList);
        Iterator i = new BindingsTable().structListIterator(list, true);
        assertNotNull(i.next());
        try {
            i.remove();
//			fail();
        } catch (UnsupportedOperationException expected) {}
    }

}
