package jTrolog.engine;

import java.util.Iterator;
import java.util.NoSuchElementException;

import junit.framework.TestCase;
import jTrolog.errors.InvalidTermException;
import jTrolog.parser.Parser;
import jTrolog.terms.Int;
import jTrolog.terms.Struct;
import jTrolog.terms.StructAtom;

/**
 * 
 * @author <a href="mailto:giulio.piancastelli@unibo.it">Giulio Piancastelli</a>
 */
public class TermIteratorTestCase extends TestCase {

    public void testEmptyIterator() throws InvalidTermException {
        String theory = "";
        Iterator i = new Parser(theory).iterator();
        assertFalse(i.hasNext());
        try {
            i.next();
            fail();
        } catch (NoSuchElementException expected) {}
    }

    public void testIterationCount() throws InvalidTermException {
        String theory = "q(1)." + "\n" +
                        "q(2)." + "\n" +
                        "q(3)." + "\n" +
                        "q(5)." + "\n" +
                        "q(7).";
        Iterator i = new Parser(theory).iterator();
        int count = 0;
        for (; i.hasNext(); count++)
            i.next();
        assertEquals(5, count);
        assertFalse(i.hasNext());
    }

    public void testMultipleHasNext() throws InvalidTermException {
        String theory = "p. q. r.";
        Iterator i = new Parser(theory).iterator();
        assertTrue(i.hasNext());
        assertTrue(i.hasNext());
        assertTrue(i.hasNext());
        assertEquals(new StructAtom("p"), i.next());
    }

    public void testMultipleNext() throws InvalidTermException {
        String theory = "p(X):-q(X),X>1." + "\n" +
                        "q(1)." + "\n" +
                        "q(2)." + "\n" +
                        "q(3)." + "\n" +
                        "q(5)." + "\n" +
                        "q(7).";
        Iterator i = new Parser(theory).iterator();
        assertTrue(i.hasNext());
        i.next(); // skip the first term
        assertEquals(new Struct("q", new Int(1)), i.next());
        assertEquals(new Struct("q", new Int(2)), i.next());
        assertEquals(new Struct("q", new Int(3)), i.next());
        assertEquals(new Struct("q", new Int(5)), i.next());
        assertEquals(new Struct("q", new Int(7)), i.next());
        // no more terms
        assertFalse(i.hasNext());
        try {
            i.next();
            fail();
        } catch (NoSuchElementException expected) {}
    }

    public void testIterationOnWrongTheory() throws InvalidTermException {
        String theory = "q(1)." + "\n" +
                        "q(2)." + "\n" +
                        "q(3) " + "\n" + // missing the End-Of-Clause!
                        "q(5)." + "\n" +
                        "q(7).";
        Iterator i = new Parser(theory).iterator();
        assertTrue(i.hasNext());
        assertEquals(new Struct("q", new Int(1)), i.next());
        assertTrue(i.hasNext());
        assertEquals(new Struct("q", new Int(2)), i.next());
        try {
            assertFalse(i.hasNext());
            System.out.println(i.next());
            fail();
        } catch (InvalidTermException expected) {}
    }

    public void testRemoveOperationNotSupported() throws InvalidTermException {
        String theory = "p(1).";
        Iterator i = new Parser(theory).iterator();
        assertNotNull(i.next());
        try {
            i.remove();
            fail();
        } catch (UnsupportedOperationException expected) {}
    }

}
