package jTrolog.engine;

import junit.framework.TestCase;
import jTrolog.errors.PrologException;

public class TheoryManagerTestCase extends TestCase {

    public void testUnknownDirective() throws PrologException {
        String theory = ":- unidentified_directive(unknown_argument).";
        Prolog engine = new Prolog();
        engine.setTheory(theory);
        assertTrue(engine.warnings.contains("The directive unidentified_directive/1 is unknown."));
    }

    public void testFailedDirective() throws PrologException {
        String theory = ":- load_library('UnknownLibrary').";
        Prolog engine = new Prolog();
        engine.setTheory(theory);
        assertTrue(engine.warnings.contains("An exception has been thrown during the execution of the load_library/1 directive.\n" +
                "InvalidLibraryException: UnknownLibrary at 0:0"));
    }

    public void testAssertNotBacktrackable() throws Throwable {
        Prolog engine = new Prolog();
        Solution firstSolution = engine.solve("assertz(a(z)).");
        assertTrue(firstSolution.success());
        assertFalse(engine.hasOpenAlternatives());
    }

    public void testRetract() throws Throwable {
        Prolog engine = new Prolog();
        engine.setTheory(
                "legs(A, 4) :- animal(A).\n" +
                "legs(octopus, 8).\n" +
                "legs(A, 6) :- insect(A).\n" +
                "legs(spider, 8).\n" +
                "legs(B, 2) :- bird(B).");
        Solution firstSolution = engine.solve("retract(legs(octopus, 8)).");
        assertTrue(firstSolution.success());
    }

    // TODO test abolish: not only the predicate is no more defined, but
    // it should also disappear completely from the clause database, i.e.
    // ClauseDatabase#get(f/a) should return null instead of an empty list
    // (it returns an empty list as a result of a retract_all, please test
    // also this one, thank you).
    //todo the above test can wait a little bit
    public void testAbolish() throws Throwable {
        Prolog engine = new Prolog();
        engine.solve("assertz(a(z)).");
        Solution result = engine.solve("a(X).");
        assertTrue(result.success());
        assertFalse(engine.hasOpenAlternatives());

        try {
            result = engine.solve("abolish(a(z)).");
            fail();
        } catch (Throwable e) {
        }
        result = engine.solve("abolish(a/1).");
        assertTrue(result.success());
        try {
            result = engine.solve("a(X).");
            fail();
        } catch (Throwable throwable) {
            assertTrue(throwable.getMessage().indexOf("a/1") > 0);
            assertTrue(throwable.getMessage().indexOf("is unknown") > 0);
        }
    }

    public void testAddTheory() {
        Prolog engine = new Prolog();
        try {
            engine.addTheory("test :- notx existing(s).");
            //Note that the engine's theory is empty, but the exception is not thrown!
            fail();
        } catch (Exception e) {
            //success
        }
    }
}
